#ifndef SSTRING_H
#define SSTRING_H

/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include <string.h>
#include <qstring.h>
#include <qregexp.h>
#include <qvaluevector.h>

namespace SubtitleComposer
{
	class SStringList;

	class SString
	{
		public:

			static const uint MaxLength = (uint)-1;

			typedef enum {
				Bold =			0x1,
				Italic =		0x2,
				Underline = 	0x4,
				StrikeThrough = 0x8,
				AllStyles =		Bold|Italic|Underline|StrikeThrough
			} StyleFlag;

			typedef enum {
				Compact,
				Verbose
			} RichOutputMode;

			SString( const QString& string=QString::null, int styleFlags=0 ):
				m_string( string ),
				m_styleFlags( 0 ),
				m_capacity( 0 )
			{
				if ( m_string.length() )
				{
					setMinFlagsCapacity( m_string.length() );
					memset( m_styleFlags, styleFlags & AllStyles, m_string.length() );
				}
			}

			SString( const SString& sstring ):
				m_string( sstring.m_string ),
				m_styleFlags( 0 ),
				m_capacity( 0 )
			{
				if ( m_string.length() )
				{
					setMinFlagsCapacity( m_string.length() );
					memcpy( m_styleFlags, sstring.m_styleFlags, m_string.length() );
				}
			}

			SString& operator=( const SString& sstring )
			{
				if ( this == &sstring )
					return *this;

				m_string = sstring.m_string;
				setMinFlagsCapacity( m_string.length() );

				if ( m_string.length() )
					memcpy( m_styleFlags, sstring.m_styleFlags, m_string.length() );

				return *this;
			}

			~SString()
			{
				delete [] m_styleFlags;
			}

			inline bool isNull() const
			{
				return m_string.isNull();
			}

			inline bool isEmpty() const
			{
				return m_string.isEmpty();
			}

			inline uint length() const
			{
				return m_string.length();
			}

			bool operator!=( const SString& sstring ) const
			{
				if ( m_string != sstring.m_string )
					return true;

				for ( int index = 0, size = m_string.length(); index < size; ++index )
					if ( m_styleFlags[index] != sstring.m_styleFlags[index] )
						return true;

				return false;
			}

			bool operator==( const SString& sstring ) const
			{
				return ! operator==( sstring );
			}

			bool operator<( const SString& sstring ) const
			{
				return m_string < sstring.m_string;
			}

			bool operator<=( const SString& sstring ) const
			{
				return m_string <= sstring.m_string;
			}

			bool operator>( const SString& sstring ) const
			{
				return m_string > sstring.m_string;
			}

			bool operator>=( const SString& sstring ) const
			{
				return m_string >= sstring.m_string;
			}

			inline QString string() const
			{
				return m_string;
			}

			/// always clears all the style flags
			void setString( const QString& string, int styleFlags=0 )
			{
				m_string = string;
				setMinFlagsCapacity( m_string.length() );
				if ( m_string.length() )
					memset( m_styleFlags, styleFlags & AllStyles, m_string.length() );
			}

			QString richString( RichOutputMode mode=Compact ) const
			{
				if ( m_string.isEmpty() )
					return m_string;

				QString ret;

				if ( mode == Compact )
				{
					char prevStyleFlags = m_styleFlags[0];
					int prevIndex = 0;

					if ( prevStyleFlags & Italic )
						ret += "<i>";
					if ( prevStyleFlags & Bold )
						ret += "<b>";
					if ( prevStyleFlags & Underline )
						ret += "<u>";
					if ( prevStyleFlags & StrikeThrough )
						ret += "<s>";

					const int size = m_string.length();
					for ( int index = 1; index < size; ++index )
					{
						if ( m_styleFlags[index] != prevStyleFlags )
						{
							QString token( m_string.mid( prevIndex, index - prevIndex ) );
							ret += token.replace( "<", "&lt;" ).replace( ">", "&gt;" );

							if ( (prevStyleFlags & StrikeThrough) && ! (m_styleFlags[index] & StrikeThrough) )
								ret += "</s>";
							if ( (prevStyleFlags & Underline) && ! (m_styleFlags[index] & Underline) )
								ret += "</u>";
							if ( (prevStyleFlags & Bold) && ! (m_styleFlags[index] & Bold) )
								ret += "</b>";
							if ( (prevStyleFlags & Italic) && ! (m_styleFlags[index] & Italic) )
								ret += "</i>";

							if ( ! (prevStyleFlags & Italic) && (m_styleFlags[index] & Italic) )
								ret += "<i>";
							if ( ! (prevStyleFlags & Bold) && (m_styleFlags[index] & Bold) )
								ret += "<b>";
							if ( ! (prevStyleFlags & Underline) && (m_styleFlags[index] & Underline) )
								ret += "<u>";
							if ( ! (prevStyleFlags & StrikeThrough) && (m_styleFlags[index] & StrikeThrough) )
								ret += "<s>";

							prevIndex = index;
							prevStyleFlags = m_styleFlags[index];
						}
					}

					QString token( m_string.mid( prevIndex, m_string.length() - prevIndex ) );
					if ( token.length() )
					{
						ret += token.replace( "<", "&lt;" ).replace( ">", "&gt;" );

						if ( prevStyleFlags & StrikeThrough )
							ret += "</u>";
						if ( prevStyleFlags & Underline )
							ret += "</u>";
						if ( prevStyleFlags & Bold )
							ret += "</b>";
						if ( prevStyleFlags & Italic )
							ret += "</i>";
					}
				}
				else // outputMode == Verbose
				{
					int currentStyleFlags = m_styleFlags[0];
					uint prevIndex = 0;
					for ( uint index = 1, size = m_string.length(); index < size; ++index )
					{
						if ( currentStyleFlags != m_styleFlags[index] )
						{
							if ( currentStyleFlags & StrikeThrough )	ret += "<s>";
							if ( currentStyleFlags & Bold )			ret += "<b>";
							if ( currentStyleFlags & Italic )		ret += "<i>";
							if ( currentStyleFlags & Underline )	ret += "<u>";

							ret += m_string.mid( prevIndex, index - prevIndex );

							if ( currentStyleFlags & Underline )	ret += "</u>";
							if ( currentStyleFlags & Italic )		ret += "</i>";
							if ( currentStyleFlags & Bold )			ret += "</b>";
							if ( currentStyleFlags & StrikeThrough )	ret += "</s>";

							prevIndex = index;

							currentStyleFlags = m_styleFlags[index];
						}
					}

					if ( prevIndex + 1 < m_string.length() )
					{
						if ( currentStyleFlags & StrikeThrough )	ret += "<s>";
						if ( currentStyleFlags & Bold )			ret += "<b>";
						if ( currentStyleFlags & Italic )		ret += "<i>";
						if ( currentStyleFlags & Underline )	ret += "<u>";

						ret += m_string.mid( prevIndex );

						if ( currentStyleFlags & Underline )	ret += "</u>";
						if ( currentStyleFlags & Italic )		ret += "</i>";
						if ( currentStyleFlags & Bold )			ret += "</b>";
						if ( currentStyleFlags & StrikeThrough )	ret += "</s>";
					}
				}

				return ret;
			}

			void setRichString( const QString& string )
			{
				QRegExp tagRegExp( "</?[bBiIuUsS]>" );

				m_string = QString::null;

				int currentStyle = 0;
				int offsetPos = 0, matchedPos;
				while ( (matchedPos = tagRegExp.search( string, offsetPos ) ) != -1 )
				{
					QString matched( tagRegExp.cap( 0 ).lower() );

					int newStyle = currentStyle;

					if ( matched == "<b>" )
						newStyle |= SString::Bold;
					else if ( matched == "<i>" )
						newStyle |= SString::Italic;
					else if ( matched == "<u>" )
						newStyle |= SString::Underline;
					else if ( matched == "<s>" )
						newStyle |= SString::StrikeThrough;
					else if ( matched == "</b>" )
						newStyle &= ~SString::Bold;
					else if ( matched == "</i>" )
						newStyle &= ~SString::Italic;
					else if ( matched == "</u>" )
						newStyle &= ~SString::Underline;
					else if ( matched == "</s>" )
						newStyle &= ~SString::StrikeThrough;

					if ( newStyle != currentStyle )
					{
						QString token( string.mid( offsetPos, matchedPos - offsetPos ) );
						append( SString( token.replace( "&lt;", "<" ).replace( "&gt;", ">" ), currentStyle ) );
						currentStyle = newStyle;
					}

					offsetPos = matchedPos + matched.length();
				}

				QString token( string.mid( offsetPos, matchedPos - offsetPos ) );
				append( SString( token.replace( "&lt;", "<" ).replace( "&gt;", ">" ), currentStyle ) );
			}


			inline int find( QChar c, int index=0, bool cs=true ) const
			{
				return m_string.find( c, index, cs );
			}

			inline int find( char c, int index=0, bool cs=true ) const
			{
				return m_string.find( c, index, cs );
			}

			inline int find( const QString& str, int index=0, bool cs=true ) const
			{
				return m_string.find( str, index, cs );
			}

			inline int find( const QRegExp& rx, int index=0 ) const
			{
				return m_string.find( rx, index );
			}

			inline int find( const char* str, int index=0 ) const
			{
				return m_string.find( str, index );
			}

			inline int findRev( QChar c, int index=-1, bool cs=true ) const
			{
				return m_string.findRev( c, index, cs );
			}

			inline int findRev( char c, int index=-1, bool cs=true ) const
			{
				return m_string.findRev( c, index, cs );
			}

			inline int findRev( const QString& str, int index=-1, bool cs=true ) const
			{
				return m_string.findRev( str, index, cs );
			}

			inline int findRev( const QRegExp& rx, int index=-1 ) const
			{
				return m_string.findRev( rx, index );
			}

			inline int findRev( const char* str, int index=-1 ) const
			{
				return m_string.findRev( str, index );
			}

			inline int contains( QChar c, bool cs=true ) const
			{
				return m_string.contains( c, cs );
			}

			inline int contains( char c, bool cs=true ) const
			{
				return m_string.contains( c, cs );
			}

			inline int contains( const char* str, bool cs=true ) const
			{
				return m_string.contains( str, cs );
			}

			inline int contains( const QString& str, bool cs=true ) const
			{
				return m_string.contains( str, cs );
			}

			inline int contains( const QRegExp& rx ) const
			{
				return m_string.contains( rx );
			}

			SString left( uint len ) const
			{
				return mid( 0, len );
			}

			SString right( uint len ) const
			{
				int index = (signed)m_string.length() - len;
				return index < 0 ? SString( *this ) : mid( index, len );
			}

			SString mid( uint index, uint len=0xffffffff ) const
			{
				if ( len > m_string.length() - index )
					len = m_string.length() - index;

				SString ret;

				if ( len )
				{
					ret.m_string = m_string.mid( index, len );
					ret.setMinFlagsCapacity( len );
					memcpy( ret.m_styleFlags, m_styleFlags + index, len );
				}

				return ret;
			}

			SString lower() const
			{
				SString ret( *this );
				ret.m_string = m_string.lower();
				return ret;
			}

			SString upper() const
			{
				SString ret( *this );
				ret.m_string = m_string.upper();
				return ret;
			}

			SString titleCase( bool lowerFirst ) const
			{
				const QString wordSeparators( " -_([:,;./\\\t\n\"" );

				SString ret( *this );

				if ( lowerFirst )
					ret.m_string = m_string.lower();

				bool wordStart = true;
				for ( uint idx = 0, size = m_string.length(); idx < size; ++idx )
				{
					QCharRef chr = ret.m_string[idx];
					if ( wordStart )
					{
						if ( ! wordSeparators.contains( chr ) )
						{
							wordStart = false;
							chr = chr.upper();
						}
					}
					else if ( wordSeparators.contains( chr ) )
						wordStart = true;
				}

				return ret;
			}

			SString sentenceCase( bool lowerFirst, bool* cont ) const
			{
				const QString sentenceEndChars( ".?!" );

				SString ret( *this );

				if ( lowerFirst )
					ret.m_string = m_string.lower();

				if ( m_string.isEmpty() )
					return ret;

				uint prevDots = 0;
				bool startSentence = cont ? ! *cont : true;

				for ( uint index = 0, size = m_string.length(); index < size; ++index )
				{
					QCharRef chr = ret.m_string[index];

					if ( sentenceEndChars.contains( chr ) )
					{
						if ( chr == '.' )
						{
							prevDots++;
							startSentence = prevDots < 3;
						}
						else
						{
							prevDots = 0;
							startSentence = true;
						}
					}
					else
					{
						if ( startSentence && chr.isLetterOrNumber() )
						{
							chr = chr.upper();
							startSentence = false;
						}

						if ( ! chr.isSpace() )
							prevDots = 0;
					}
				}

				if ( cont )
					*cont = prevDots != 1 && ! startSentence;

				return ret;
			}

			SString stripWhiteSpace() const
			{
				const QRegExp stripSpaceRegExp( "(^\\s+|\\s+$)" );

				SString ret( *this );
				return ret.replace( stripSpaceRegExp, "" );
			}


			inline SString simplifyWhiteSpace() const
			{
				const QRegExp simplifySpaceRegExp( "\\s{2,MAXINT}" );

				return stripWhiteSpace().replace( simplifySpaceRegExp, "" );
			}

			SString& insert( uint index, const QString& str )
			{
				if ( str.length() )
				{
					uint origLength = m_string.length();

					m_string.insert( index, str );

					uint addedLength = m_string.length() - origLength;

					char* oldStyleFlags = detachFlags();
					setMinFlagsCapacity( m_string.length() );

					char fillFlags = 0;
					if ( origLength )
					{
						if ( index == 0 )
							fillFlags = oldStyleFlags[0];
						else if ( index < origLength )
							fillFlags = oldStyleFlags[index-1];
						else // ( index >= origLength )
							fillFlags = oldStyleFlags[origLength-1];
					}

					memcpy( m_styleFlags, oldStyleFlags, index );
					memset( m_styleFlags + index, fillFlags, addedLength );
					memcpy( m_styleFlags + index + addedLength, oldStyleFlags + index, m_string.length() - index - addedLength );

					delete [] oldStyleFlags;
				}

				return *this;
			}

			SString& insert( uint index, const SString& str )
			{
				if ( str.m_string.length() )
				{
					uint origLength = m_string.length();

					m_string.insert( index, str.m_string );

					uint addedLength = m_string.length() - origLength;

					char* oldStyleFlags = detachFlags();
					setMinFlagsCapacity( m_string.length() );

					memcpy( m_styleFlags, oldStyleFlags, index );

					if ( addedLength > str.m_string.length() )
					{
						// NOTE: origLength must be > 0 in this case
						uint fillLength = addedLength - str.m_string.length();
						memset( m_styleFlags + index, oldStyleFlags[origLength-1], fillLength );
						memcpy( m_styleFlags + index + fillLength, str.m_styleFlags, addedLength - fillLength );
					}
					else // addedLength == str.m_string.length()
						memcpy( m_styleFlags + index, str.m_styleFlags, addedLength );

					memcpy( m_styleFlags + index + addedLength, oldStyleFlags + index, m_string.length() - index - addedLength );

					delete [] oldStyleFlags;
				}

				return *this;
			}

			inline SString& append( const QString& str )
			{
				return insert( m_string.length(), str );
			}

			inline SString& append( const SString& str )
			{
				return insert( m_string.length(), str );
			}

			inline SString& prepend( const QString& str )
			{
				return insert( 0, str );
			}

			inline SString& prepend( const SString& str )
			{
				return insert( 0, str );
			}

			inline SString& remove( uint index, uint len )
			{
				return replace( index, len, "" );
			}

			inline SString& remove( const QString& str, bool cs=true )
			{
				return replace( str, "", cs );
			}

			inline SString& remove( const QRegExp& rx )
			{
				return replace( rx, "" );
			}


			SString& replace( uint index, uint len, const QString& str )
			{
				if ( index + len > m_string.length() )
					len = m_string.length() - index;

				if ( ! len )
				{
					return *this;
				}
				else 
				{
					m_string.replace( index, len, str );

					// simple path for when there's no need to change the styles
					if ( len == 1 && str.length() == 1 )
						return *this;

					if ( len == str.length() )
					{
						memset( m_styleFlags + index, m_styleFlags[index], str.length() );
					}
					else
					{
						char* oldStyleFlags = detachFlags();
						setMinFlagsCapacity( m_string.length() );

						memcpy(
							m_styleFlags,
							oldStyleFlags,
							index
						);

						memset(
							m_styleFlags + index,
							oldStyleFlags[index],
							str.length()
						);

						memcpy(
							m_styleFlags + index + str.length(),
							oldStyleFlags + index + len,
							m_string.length() - index - str.length()
						);

						delete [] oldStyleFlags;
					}

					return *this;
				}
			}

			SString& replace( const QString& before, const QString& after, bool cs=true )
			{
				if ( before.isEmpty() )
				{
					return *this;
				}
				else if ( before.length() == 1 && after.length() == 1 )
				{
					// simple path for when there's no need to change the styles
					m_string.replace( before, after );
					return *this;
				}
				else
				{
					uint origLength = m_string.length();

					QValueVector<int> changedData;

					for ( int offsetIndex = 0, matchedIndex; (matchedIndex = m_string.find( before, offsetIndex, cs )) != -1; offsetIndex = matchedIndex + after.length() )
					{
						m_string.replace( matchedIndex, before.length(), after );

						changedData.append( matchedIndex );
					}

					if ( changedData.empty() ) // nothing was replaced
						return *this;

					if ( m_string.length() )
					{
						uint beforeLength = before.length(), afterLength = after.length();
						uint srcOffset = 0, dstOffset = 0;
						uint srcTokenSize;

						char* oldStyleFlags = detachFlags();
						setMinFlagsCapacity( m_string.length() );

						for ( uint index = 0; index < changedData.size(); index += 3 )
						{
							srcTokenSize = changedData[index] - srcOffset - 1;

							memcpy( m_styleFlags + dstOffset, oldStyleFlags + srcOffset, srcTokenSize );

							dstOffset += srcTokenSize;
							memset( m_styleFlags + dstOffset, m_styleFlags[srcOffset], afterLength );
							dstOffset += afterLength;

							srcOffset += srcTokenSize;
							srcOffset += beforeLength;
						}

						memcpy( m_styleFlags + dstOffset, oldStyleFlags + srcOffset, origLength - srcOffset - 1 );

						delete [] oldStyleFlags;
					}
					else
						setMinFlagsCapacity( m_string.length() );

					return *this;
				}
			}

			SString& replace( const QRegExp& rx, const QString& a )
			{
				QRegExp regExp( rx );

				uint origLength = m_string.length();

				QValueVector<int> changedData;

				for ( int offsetIndex = 0, matchedIndex; (matchedIndex = regExp.search( m_string, offsetIndex )) != -1; )
				{
					QString after( a );

					bool escaping = false;
					for ( int afterIndex = 0, afterSize = after.length(); afterIndex < afterSize; ++afterIndex )
					{
						QChar chr = after.at( afterIndex );
						if ( escaping && chr.isNumber() ) // perform replace
						{
							int capNumber = chr.digitValue();
							if ( capNumber <= regExp.numCaptures() )
							{
								QString cap( regExp.cap( capNumber ) );
								after.replace( afterIndex - 1, 2, cap );
								afterIndex = afterIndex - 2 + cap.length();
								afterSize = after.length();
							}
						}
						else if ( chr == '\\' )
							escaping = ! escaping;
					}

					m_string.replace( matchedIndex, regExp.matchedLength(), after );

					if ( regExp.matchedLength() != 1 || after.length() != 1 )
					{
						changedData.append( matchedIndex );
						changedData.append( regExp.matchedLength() ); // before length
						changedData.append( after.length() ); // after length
					}

					offsetIndex = matchedIndex + after.length();
				}

				if ( changedData.empty() ) // nothing was replaced (or there's no need to touch the style flags)
					return *this;

				if ( m_string.length() )
				{
					uint beforeLength, afterLength;
					uint srcOffset = 0, dstOffset = 0;
					uint srcTokenSize;

					char* oldStyleFlags = detachFlags();
					setMinFlagsCapacity( m_string.length() );

					for ( uint index = 0; index < changedData.size(); index += 3 )
					{
						srcTokenSize = changedData[index] - srcOffset - 1;
						beforeLength = changedData[index+1];
						afterLength = changedData[index+2];

						memcpy( m_styleFlags + dstOffset, oldStyleFlags + srcOffset, srcTokenSize );

						dstOffset += srcTokenSize;
						memset( m_styleFlags + dstOffset, m_styleFlags[srcOffset], afterLength );
						dstOffset += afterLength;

						srcOffset += srcTokenSize;
						srcOffset += beforeLength;
					}

					memcpy( m_styleFlags + dstOffset, oldStyleFlags + srcOffset, origLength - srcOffset - 1 );

					delete [] oldStyleFlags;
				}
				else
					setMinFlagsCapacity( 0 );

				return *this;
			}

			inline SString& operator+=( const QString& str )
			{
				return append( str );
			}

			inline SString& operator+=( const SString& str )
			{
				return append( str );
			}

			inline QChar at( uint i ) const
			{
				return m_string.at( i );
			}

			inline QChar operator[]( int i ) const
			{
				return m_string[i];
			}

			inline QCharRef at( uint i )
			{
				if ( i >= m_string.length() )
				{
					unsigned fillerLength = m_string.length() - i + 1;
					char filler[fillerLength];
					memset( filler, ' ', fillerLength - 1 );
					filler[fillerLength-1] = 0;
					append( QString( filler ) );
				}

				return m_string.at( i );
			}

			inline QCharRef operator[]( int i )
			{
				if ( (unsigned)i >= m_string.length() )
				{
					unsigned fillerLength = m_string.length() - i + 1;
					char filler[fillerLength];
					memset( filler, ' ', fillerLength - 1 );
					filler[fillerLength-1] = 0;
					append( QString( filler ) );
				}

				return m_string[i];
			}

			inline int styleFlagsAt( uint i ) const
			{
				return i >= m_string.length() ? 0 : m_styleFlags[i];
			}

			inline void setStyleFlagsAt( uint i, int styleFlags ) const
			{
				if ( i < m_string.length() )
					m_styleFlags[i] = styleFlags;
			}

			int cumStyleFlags() const
			{
				int cumStyleFlags = 0;
				for ( uint index = 0, size = m_string.length(); index < size; ++index )
				{
					cumStyleFlags |= m_styleFlags[index];
					if ( cumStyleFlags == AllStyles )
						break;
				}
				return cumStyleFlags;
			}

			bool hasStyleFlags( int styleFlags ) const
			{
				int cumStyleFlags = 0;
				for ( uint index = 0, size = m_string.length(); index < size; ++index )
				{
					cumStyleFlags |= m_styleFlags[index];
					if ( (cumStyleFlags & styleFlags) == styleFlags )
						return true;
				}
				return false;
			}

			SString& setStyleFlags( uint index, uint len, int styleFlags, bool on )
			{
				if ( len == 0 || index >= m_string.length() )
					return *this;

				uint size = index + len;
				if ( size > m_string.length() )
					size = m_string.length();

				if ( on )
				{
					for ( ; index < size; ++index )
						m_styleFlags[index] = m_styleFlags[index] | styleFlags;
				}
				else
				{
					styleFlags = ~styleFlags;
					for ( ; index < size; ++index )
						m_styleFlags[index] = m_styleFlags[index] & styleFlags;
				}

				return *this;
			}

			SString& setStyleFlags( uint index, uint len, int styleFlags )
			{
				if ( len == 0 || index >= m_string.length() )
					return *this;

				uint size = index + len;
				if ( size > m_string.length() )
					size = m_string.length();

				for ( ; index < size; ++index )
					m_styleFlags[index] = styleFlags;

				return *this;
			}

		private:

			inline char* detachFlags()
			{
				char* ret = m_styleFlags;
				m_styleFlags = 0;
				m_capacity = 0;
				return ret;
			}

			inline void setMinFlagsCapacity( uint capacity )
			{
				if ( capacity > m_capacity )
				{
					m_capacity = capacity * 2;
					delete [] m_styleFlags;
					m_styleFlags = new char[m_capacity];
				}
				else if ( capacity == 0 )
				{
					m_capacity = 0;
					delete [] m_styleFlags;
					m_styleFlags = 0;
				}
				else if ( m_capacity > 100 && capacity < m_capacity / 2 )
				{
					m_capacity = m_capacity / 2;
					delete [] m_styleFlags;
					m_styleFlags = new char[m_capacity];
				}
			}

		private:

			QString m_string;
			char* m_styleFlags;
			uint m_capacity;
	};

	class SStringList : public QValueList<SString>
	{
		public:

			SStringList()
			{
			}

			SStringList( const SString& str )
			{
				append( str );
			}

			SStringList( const SStringList& list ):
				QValueList<SString>( list )
			{
			}

			SStringList( const QValueList<SString>& list ):
				QValueList<SString>( list )
			{
			}

			SStringList( const QStringList& list )
			{
				for ( QStringList::ConstIterator it = list.begin(), end = list.end(); it != end; ++it )
					append( *it );
			}

			SStringList( const QValueList<QString>& list )
			{
				for ( QValueList<QString>::ConstIterator it = list.begin(), end = list.end(); it != end; ++it )
					append( *it );
			}

			void sort()
			{
			    qHeapSort( *this );
			}

			static SStringList split( const QString& sep, const SString& str, bool keepEmpty=false, bool cs=true )
			{
				SStringList ret;

				if ( sep.length() )
				{
					int offsetIndex = 0;

					for ( int matchedIndex;
						  (matchedIndex = str.find( sep, offsetIndex, cs )) != -1;
						  offsetIndex = matchedIndex + sep.length() )
					{
						SString token( str.mid( offsetIndex, matchedIndex - offsetIndex ) );
						if ( keepEmpty || token.length() )
							ret << token;
					}

					SString token( str.mid( offsetIndex ) );
					if ( keepEmpty || token.length() )
						ret << token;
				}

				return ret;
			}

			static SStringList split( const QChar& sep, const SString& str, bool keepEmpty=false, bool cs=true )
			{
			    return split( QString( sep ), str, keepEmpty, cs );
			}

			static SStringList split( const QRegExp& sep, const SString& str, bool keepEmpty=false )
			{
				SStringList ret;

				QRegExp sepAux( sep );
				QString strAux( str.string() );

				int offsetIndex = 0;

				for ( int matchedIndex;
					  (matchedIndex = sepAux.search( strAux, offsetIndex )) != -1;
					  offsetIndex = matchedIndex + sepAux.matchedLength() )
				{
					SString token( str.mid( offsetIndex, matchedIndex - offsetIndex ) );
					if ( keepEmpty || token.length() )
						ret << token;
				}

				SString token( str.mid( offsetIndex ) );
				if ( keepEmpty || token.length() )
					ret << token;

				return ret;
			}

			SString join( const SString& sep ) const
			{
				SString ret;

				bool skipSeparator = true;
				for ( SStringList::ConstIterator it = begin(), end = this->end(); it != end; ++it )
				{
					if ( skipSeparator )
					{
						ret += *it;
						skipSeparator = false;
						continue;
					}
					ret += sep;
					ret += *it;
				}

				return ret;
			}

		protected:

			friend class QDeepCopy<SStringList>;
	};
}

#endif
