/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "lineswidget.h"
#include "lineswidgetitem.h"

#include <subtitle.h>
#include <subtitleline.h>
#include <timevalidator.h>

#include <qpainter.h>
#include <qheader.h>
#include <qsimplerichtext.h>
#include <qstylesheet.h>

#include <kdebug.h>
#include <kapplication.h>
#include <kiconloader.h>

using namespace SubtitleComposer;

LinesWidgetItem::LinesWidgetItem( SubtitleLine* line, LinesWidget* parent, LinesWidgetItem* after ):
	QObject( 0 ),
	KListViewItem( parent, after ),
	m_line( line ),
	m_isPlaying( false )
{
	connect( line, SIGNAL( primaryTextChanged(const SString&) ), this, SLOT( updatePrimaryText(const SString&) ) );
	connect( line, SIGNAL( secondaryTextChanged(const SString&) ), this, SLOT( updateSecondaryText(const SString&) ) );
	connect( line, SIGNAL( showTimeChanged(const Time&) ), this, SLOT( updateShowTime(const Time&) ) );
	connect( line, SIGNAL( hideTimeChanged(const Time&) ), this, SLOT( updateHideTime(const Time&) ) );
	connect( line, SIGNAL( errorFlagsChanged(int) ), this, SLOT( updateErrors(int) ) );
	connect( line, SIGNAL( markChanged(bool) ), this, SLOT( updateMark(bool) ) );

	updatePrimaryText( line->primaryText() );
	updateSecondaryText( line->secondaryText() );
	updateShowTime( line->showTime() );
	updateHideTime( line->hideTime() );
	updateErrors( line->errorFlags() );
	updateMark( line->marked() );
}

LinesWidgetItem::~LinesWidgetItem()
{
}

const QPixmap& LinesWidgetItem::markPixmap()
{
	static QPixmap errorPixmap;

	if ( errorPixmap.isNull() )
		errorPixmap = kapp->iconLoader()->loadIcon( "messagebox_warning", KIcon::Desktop, 13 );

	return errorPixmap;
}

const QPixmap& LinesWidgetItem::errorPixmap()
{
	static QPixmap errorPixmap;

	if ( errorPixmap.isNull() )
		errorPixmap = kapp->iconLoader()->loadIcon( "messagebox_critical", KIcon::Desktop, 13 );

	return errorPixmap;
}

SubtitleLine* LinesWidgetItem::line()
{
	return m_line;
}

bool LinesWidgetItem::isPlaying()
{
	return m_isPlaying;
}

void LinesWidgetItem::setPlaying( bool playing )
{
	if ( m_isPlaying != playing )
	{
		m_isPlaying = playing;
		repaint();
	}
}

LinesWidgetItem* LinesWidgetItem::nextSiblingCasted()
{
	return static_cast<LinesWidgetItem*>( nextSibling() );
}

int LinesWidgetItem::width( const QFontMetrics& fm, const QListView* lv, int c ) const
{
	return KListViewItem::width( fm, lv, c ) + 10;
}

void LinesWidgetItem::updateNumber( unsigned number )
{
	setText( LinesWidget::Number, QString::number( number ) );
}

void LinesWidgetItem::updatePrimaryText( const SString& text )
{
	setText( LinesWidget::Text, text.richString().replace( "\n", "|" ) );
}

void LinesWidgetItem::updateSecondaryText( const SString& text )
{
	setText( LinesWidget::Translation, text.richString().replace( "\n", "|" ) );
}

void LinesWidgetItem::updateShowTime( const Time& showTime )
{
	setText( LinesWidget::ShowTime, showTime.toString() );
}

void LinesWidgetItem::updateHideTime( const Time& hideTime )
{
	setText( LinesWidget::HideTime, hideTime.toString() );
}

void LinesWidgetItem::updateErrors( int )
{
	repaint();
}

void LinesWidgetItem::updateMark( bool )
{
	repaint();
}

QStyleSheet* LinesWidgetItem::styleSheet()
{
	static QStyleSheet* styleSheet( 0 );

	if ( ! styleSheet )
	{
		styleSheet = new QStyleSheet();
		QStyleSheetItem* pItem = new QStyleSheetItem( styleSheet, "p" );
		pItem->setDisplayMode( QStyleSheetItem::DisplayInline );
		// pItem->setMargin( QStyleSheetItem::MarginVertical, 0 );
		pItem->setVerticalAlignment( QStyleSheetItem::VAlignBaseline );
		pItem->setWhiteSpaceMode( QStyleSheetItem::WhiteSpacePre );
	}

	return styleSheet;
}

void LinesWidgetItem::paintCell( QPainter* painter, const QColorGroup& cg, int column, int width, int align )
{
	int height = this->height();

	painter->fillRect(
		0, 0,
		width,
		height,
		m_isPlaying ?
			(isSelected() ? cg.highlight().light( 140 ) : cg.highlight().light( 165 )) :
			(isSelected() ? cg.highlight() : cg.base())
	);

	QPen pen;
	pen.setWidth( 1 );
	pen.setColor( cg.mid().light( 110 ) );
	pen.setStyle( Qt::DotLine );
	painter->setPen( pen );

	for ( int x = 0; x < width; x += 2 )
		painter->drawPoint( x, height-1 );

	if ( column )
	{
		for ( int y = 1; y < height; y += 2 )
			painter->drawPoint( 0, y );
	}

	pen.setStyle( Qt::SolidLine );
	pen.setColor( isSelected() ? cg.highlightedText() : cg.text() );
	painter->setPen( pen );

	int xoffset = align & Qt::AlignHCenter ? 2 : 7;

	if ( column >= LinesWidget::Text )
	{
		if ( m_line->marked() )
		{
			painter->drawPixmap( xoffset, 1, markPixmap() );
			xoffset += 16;
		}

		int errorFlags = m_line->errorFlags();
		if ( (errorFlags & (SubtitleLine::OtherErrors|SubtitleLine::TimesErrors)) ||
			 (column == LinesWidget::Text && (errorFlags & SubtitleLine::PrimaryTextErrors)) ||
			 (column == LinesWidget::Translation && (errorFlags & SubtitleLine::SecondaryTextErrors)) )
		{
			painter->drawPixmap( xoffset, 1, errorPixmap() );
			xoffset += 16;
		}

		QString text(
			column == LinesWidget::Text ?
				m_line->primaryText().richString( SString::Verbose ) :
				m_line->secondaryText().richString( SString::Verbose )
		);

		QSimpleRichText richText( "<p>" + text.replace( "\n", "|" ) + "</p>", painter->font(), QString::null, styleSheet() );
		richText.setWidth( width - 2 );
		richText.draw( painter, xoffset, -3 /*yoffset*/, QRect(), cg );
	}
	else
		painter->drawText( xoffset, 0 /*yoffset*/, width-2, height, align, text( column ) );
}

void LinesWidgetItem::okRename( int column )
{
	int columnWidth = listView()->columnWidth( column );
	KListViewItem::okRename( column );

	QString text( this->text( column ) );

	if ( column == LinesWidget::Text )
	{
		SString stext;
		stext.setRichString( text.replace( "|", "\n" ) );
		m_line->setPrimaryText( stext );
	}
	else if ( column == LinesWidget::Translation )
	{
		SString stext;
		stext.setRichString( text.replace( "|", "\n" ) );
		m_line->setSecondaryText( stext );
	}
	else
	{
		TimeValidator timeValidator;

		int timeMillis;
		if ( column == LinesWidget::ShowTime )
		{
			if ( timeValidator.parse( text, timeMillis ) )
				m_line->setShowTime( timeMillis );
			else
			{
				updateShowTime( m_line->showTime() );
				listView()->setColumnWidth( column, columnWidth );
			}
		}
		else if ( column == LinesWidget::HideTime )
		{
			if ( timeValidator.parse( text, timeMillis ) )
				m_line->setHideTime( timeMillis );
			else
			{
				updateHideTime( m_line->hideTime() );
				listView()->setColumnWidth( column, columnWidth );
			}
		}
	}

	// items are only editable when we say so
	setRenameEnabled( LinesWidget::ShowTime, false );
	setRenameEnabled( LinesWidget::HideTime, false );
	setRenameEnabled( LinesWidget::Text, false );
}

void LinesWidgetItem::cancelRename( int column )
{
	KListViewItem::cancelRename( column );

	// items are only editable when we say so
	setRenameEnabled( LinesWidget::ShowTime, false );
	setRenameEnabled( LinesWidget::HideTime, false );
	setRenameEnabled( LinesWidget::Text, false );
}

#include "lineswidgetitem.moc"
