/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "textoverlaywidget.h"

#include <qdatetime.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qbitmap.h>
#include <qpainter.h>
#include <qsimplerichtext.h>
#include <qstylesheet.h>

#include <kdebug.h>

TextOverlayWidget::TextOverlayWidget( QWidget* parent ):
	QWidget( parent, 0, 0 ),
	m_text(),
	m_font(),
	m_primaryColor( Qt::yellow ),
	m_primaryRGB( m_primaryColor.rgb() ),
	m_outlineWidth( 1 ),
	m_outlineColor( Qt::black ),
	m_outlineRGB( m_outlineColor.rgb() ),
	m_transColor( 0, 0, 0 ),
	m_transRGB( m_transColor.rgb() ),
	m_richText( 0 ),
	m_alignment( Qt::AlignVCenter|Qt::AlignHCenter ),
	m_dirty( true )
{
	setWFlags( Qt::WNoAutoErase );

	m_font.setPointSize( 15 );

	m_styleSheet = new QStyleSheet();
	QStyleSheetItem* styleSheetItem = new QStyleSheetItem( m_styleSheet, "p" );
	styleSheetItem->setDisplayMode( QStyleSheetItem::DisplayBlock );
	styleSheetItem->setWhiteSpaceMode( QStyleSheetItem::WhiteSpacePre );
	styleSheetItem->setAlignment( m_alignment );

	updateColors();
}

TextOverlayWidget::~TextOverlayWidget()
{
	delete m_richText;
}

void TextOverlayWidget::setDirty( bool updateRichText, bool updateColors )
{
	if ( updateRichText && m_richText )
	{
		delete m_richText;
		m_richText = new QSimpleRichText( "<p>" + m_text + "</p>", m_font, QString::null, m_styleSheet );
		m_richText->setWidth( width() );

		QStyleSheetItem* styleSheetItem = m_styleSheet->item( "p" );
		styleSheetItem->setFontSize( m_font.pointSize() );
		styleSheetItem->setAlignment( m_alignment );
	}

	if ( updateColors )
		this->updateColors();

	m_dirty = true;
	update();
}

QString TextOverlayWidget::text() const
{
	return m_text;
}

void TextOverlayWidget::setText( const QString& text )
{
	if ( m_text != text )
	{
		m_text = text;
		setDirty( true, false );
	}
}

bool TextOverlayWidget::richTextMode() const
{
	return m_richText;
}

void TextOverlayWidget::setRichTextMode( bool value )
{
	if ( value && ! m_richText )
	{
		m_richText = new QSimpleRichText( "<p>" + m_text + "</p>", m_font, QString::null, m_styleSheet );
		m_richText->setWidth( width() );
	}
	else if ( ! value && m_richText )
	{
		delete m_richText;
		m_richText = 0;
	}
}

int TextOverlayWidget::alignment() const
{
	return m_alignment;
}

void TextOverlayWidget::setAlignment( int alignment )
{
	if ( m_alignment != alignment )
	{
		m_alignment = alignment;
		setDirty( true, false );
	}
}

int TextOverlayWidget::pointSize() const
{
	return m_font.pointSize();
}

void TextOverlayWidget::setPointSize( int pointSize )
{
	if ( m_font.pointSize() != pointSize )
	{
		m_font.setPointSize( pointSize );
		setDirty( true, false );
	}
}

float TextOverlayWidget::pointSizeFloat() const
{
	return m_font.pointSizeFloat();
}

void TextOverlayWidget::setPointSizeFloat( float pointSizeFloat )
{
	if ( m_font.pointSizeFloat() != pointSizeFloat )
	{
		m_font.setPointSizeFloat( pointSizeFloat );
		setDirty( true, false );
	}
}

int TextOverlayWidget::pixelSize() const
{
	return m_font.pixelSize();
}

void TextOverlayWidget::setPixelSize( int pixelSize )
{
	if ( m_font.pixelSize() != pixelSize )
	{
		m_font.setPixelSize( pixelSize );
		setDirty( true, false );
	}
}

QString TextOverlayWidget::family() const
{
	return m_font.family();
}

void TextOverlayWidget::setFamily( const QString& family )
{
	if ( m_font.family() != family )
	{
		m_font.setFamily( family );
		setDirty( true, false );
	}
}

QColor TextOverlayWidget::primaryColor() const
{
	return m_primaryColor;
}

void TextOverlayWidget::setPrimaryColor( const QColor& color )
{
	if ( m_primaryColor != color )
	{
		m_primaryColor = color;
		setDirty( false, true );
	}
}

unsigned TextOverlayWidget::outlineWidth() const
{
	return m_outlineWidth;
}

void TextOverlayWidget::setOutlineWidth( unsigned width )
{
	if ( m_outlineWidth != width )
	{
		m_outlineWidth = width;
		setDirty( false, false );
	}
}

QColor TextOverlayWidget::outlineColor() const
{
	return m_outlineColor;
}

void TextOverlayWidget::setOutlineColor( const QColor& color )
{
	if ( m_outlineColor != color )
	{
		m_outlineColor = color;
		setDirty( false, true );
	}
}

void TextOverlayWidget::paintEvent( QPaintEvent* /*event*/ )
{
// 	static QTime time;
// 	time.start();

	if ( m_dirty )
		updateContents();

	if ( ! m_text.isEmpty() )
	{
		QPainter( this ).drawPixmap( 0, 0, m_bgPixmap );
	}

// 	kdDebug() << "paintEvent " << time.elapsed() << endl;
}

void TextOverlayWidget::resizeEvent( QResizeEvent* e )
{
	if ( m_richText )
		m_richText->setWidth( e->size().width() );

	m_noTextMask = QBitmap( e->size().width(), e->size().height(), true );
	m_bgPixmap = QPixmap( e->size() );

	m_dirty = true;
}

void TextOverlayWidget::updateColors()
{
	// the outline drawing algorithm expect different primary and outline colors
	// so we have to change one if they are the same
	if ( m_outlineColor == m_primaryColor )
	{
		QRgb rgb = m_outlineColor.rgb();
		int blue = qBlue( rgb );
		m_outlineColor.setRgb( qRed( rgb ), qGreen( rgb ), blue > 0 ? blue - 1 : blue + 1 );
	}

	// the primary and outline colors can't be the same as the translucent color
	// or they won't be shown. It's also best if the translucent color is as dark
	// as possible.
	for ( int blue = 0; m_primaryColor == m_transColor || m_outlineColor == m_transColor; ++blue )
		m_transColor.setRgb( 0, 0, blue );

	m_primaryRGB = m_primaryColor.rgb();
	m_outlineRGB = m_outlineColor.rgb();
	m_transRGB = m_transColor.rgb();
}

void TextOverlayWidget::updateContents()
{
	if ( m_text.isEmpty() )
	{
		setMask( m_noTextMask );
	}
	else
	{
		QPainter painter( &m_bgPixmap );

		QRect rect( this->rect() );

		painter.fillRect( rect, m_transColor );
		painter.setFont( m_font );

		if ( m_richText )
		{
			const QRect nullRect;

			QColorGroup colorGroup( this->colorGroup() );
			colorGroup.setColor( QColorGroup::Text, m_primaryColor );

			int textHeight = m_richText->height(), yoffset;
			if ( m_alignment & Qt::AlignBottom )
				yoffset = rect.height() - textHeight;
			else if ( m_alignment & AlignTop )
				yoffset = 0;
			else // if ( m_alignment & AlignVCenter || m_alignment & AlignCenter )
				yoffset = (rect.height() - textHeight) / 2;

			m_richText->draw( &painter, 0, yoffset, nullRect, colorGroup );

			if ( textHeight > rect.height() )
			{
				yoffset = 0;
				textHeight = rect.height();
			}

			int textWidth = m_richText->widthUsed(), xoffset;
			if ( textWidth > rect.width() )
			{
				xoffset = 0;
				textWidth = rect.width();
			}
			else
			{
				if ( m_alignment & Qt::AlignLeft )
					xoffset = 0;
				else if ( m_alignment & AlignRight )
					xoffset = rect.width() - textWidth;
				else // if ( m_alignment & AlignHCenter || m_alignment & AlignCenter )
					xoffset = (rect.width() - textWidth) / 2;
			}

			QRect textRect( xoffset, yoffset, textWidth, textHeight );
			setMaskAndOutline( textRect, m_outlineWidth );
		}
		else
		{
			painter.setPen( m_primaryColor );
			painter.drawText( rect, m_alignment, m_text );

			QRect textRect( painter.boundingRect( rect, m_alignment, m_text ) );
			textRect |= rect;
			setMaskAndOutline( textRect, m_outlineWidth );
		}
	}

	m_dirty = false;
}

void TextOverlayWidget::setMaskAndOutline( QRect& textRect, unsigned outlineWidth )
{
	static const QRgb color0 = Qt::color0.rgb();
	static const QRgb color1 = Qt::color1.rgb();

	QImage bgImage = m_bgPixmap.convertToImage();

	// NOTE: we use a 32 bits image for the mask creation for performance reasons
	QImage maskImage( size(), 32 /*depth*/, 2 /*numColors*/, QImage::LittleEndian );
	maskImage.fill( color0 );
	QBitmap maskBitmap;

	if ( outlineWidth )
	{
		int width = this->width();
		int maxImageX = width - 1, maxImageY = height() - 1;

// 		int lminX, lmaxX, lminY, lmaxY;
// 		for ( int y = textRect.top(), maxY = textRect.bottom(); y <= maxY; ++y )
// 		{
// 			for ( int x = textRect.left(), maxX = textRect.right(); x <= maxX; ++x )
// 			{
// 				QRgb pixel = image.pixel( x, y );
//
// 				if ( pixel == m_primaryRGB ) // draw the outline
// 				{
// 					lminY = y - outlineWidth;
// 					if ( lminY < 0 ) lminY = 0;
// 					lmaxY = y + outlineWidth;
// 					if ( lmaxY > maxImageY ) lmaxY = maxImageY;
// 					lminX = x - outlineWidth;
// 					if ( lminX < 0 ) lminX = 0;
// 					lmaxX = x + outlineWidth;
// 					if ( lmaxX > maxImageX ) lmaxX = maxImageX;
//
// 					for ( int ly = lminY; ly <= lmaxY; ++ly )
// 					{
// 						for ( int lx = lminX; lx <= lmaxX; ++lx )
// 						{
// 							maskImage.setPixel( lx, ly, color1 );
// 							if ( image.pixel( lx, ly ) != m_primaryRGB )
// 								image.setPixel( lx, ly, m_outlineRGB );
// 						}
// 					}
// 				}
// 			}
// 		}

		QRgb* bgImageScanLines[maxImageY+1];
		QRgb* maskImageScanLines[maxImageY+1];

		int outlineMinY = textRect.top() - outlineWidth;
		if ( outlineMinY < 0 )
			outlineMinY = 0;

		int outlineMaxY = textRect.bottom() + outlineWidth;
		if ( outlineMaxY > maxImageY )
			outlineMaxY = maxImageY;

// 		static QTime time;
// 		time.start();
// 		// we don't need all the values initialized, only the ones we will actually use
		for ( int y = outlineMinY; y <= outlineMaxY; ++y )
		{
			bgImageScanLines[y] = (QRgb*)bgImage.scanLine( y );
			maskImageScanLines[y] = (QRgb*)maskImage.scanLine( y );
		}
// 		QRgb* bgImageBits = (QRgb*)bgImage.bits() + outlineMinY*width;
// 		QRgb* maskImageBits = (QRgb*)maskImage.bits() + outlineMinY*width;
// 		for ( int y = outlineMinY; y <= outlineMaxY; ++y )
// 		{
// 			bgImageScanLines[y] = bgImageBits;
// 			maskImageScanLines[y] = maskImageBits;
// 			bgImageBits += width;
// 			maskImageBits += width;
// 		}
// 		kdDebug() << time.elapsed() << endl;

		int lminX, lmaxX, lminY, lmaxY;
		for ( int y = textRect.top(), maxY = textRect.bottom(); y <= maxY; ++y )
		{
			for ( int x = textRect.left(), maxX = textRect.right(); x <= maxX; ++x )
			{
				if ( bgImageScanLines[y][x] == m_primaryRGB ) // draw the outline
				{
					lminY = y - outlineWidth;
					if ( lminY < 0 ) lminY = 0;
					lmaxY = y + outlineWidth;
					if ( lmaxY > maxImageY ) lmaxY = maxImageY;
					lminX = x - outlineWidth;
					if ( lminX < 0 ) lminX = 0;
					lmaxX = x + outlineWidth;
					if ( lmaxX > maxImageX ) lmaxX = maxImageX;

					for ( int ly = lminY; ly <= lmaxY; ++ly )
					{
						for ( int lx = lminX; lx <= lmaxX; ++lx )
						{
							maskImageScanLines[ly][lx] = color1;
							if ( bgImageScanLines[ly][lx] != m_primaryRGB )
								bgImageScanLines[ly][lx] = m_outlineRGB;
						}
					}
				}
			}
		}

		m_bgPixmap.convertFromImage( bgImage );
	}
	else
	{
		QRgb* bgImageBits = (QRgb*)bgImage.bits();
		QRgb* maskImageBits = (QRgb*)maskImage.bits();

		for ( int index = 0, size = width()*height(); index < size; ++index )
		{
			if ( bgImageBits[index] != m_transRGB )
				maskImageBits[index] = color1;
		}
	}

	maskBitmap.convertFromImage( maskImage, Qt::MonoOnly );

	// setBackgroundPixmap( m_bgPixmap );
// 	setErasePixmap( m_bgPixmap );

	setMask( maskBitmap );
}

QSize TextOverlayWidget::minimumSizeHint() const
{
	if ( m_richText )
		return QSize( m_richText->widthUsed(), m_richText->height() );
	else
	{
		QPainter p( this );
		QRect rect = p.boundingRect( 0, 0, 300, 300, m_alignment, m_text );
		return rect.size();
	}
}
